#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom exceptions
"""


class CachedPage(Exception):
    """
    This error is raised when archive.org declines to make a new capture
    and instead returns the cached version of most recent archive.
    """
    pass


class WaybackRuntimeError(Exception):
    """
    An error returned by the Wayback Machine.
    """
    pass


class BlockedByRobots(WaybackRuntimeError):
    """
    This error is raised when archive.org has been blocked by the site's robots.txt access control instructions.
    """
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .api import capture, capture_or_cache, CachedPage, BlockedByRobots


__version__ = "0.0.13"

__all__ = (
    'BlockedByRobots',
    'capture',
    'capture_or_cache',
    'CachedPage'
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from setuptools import setup


def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read()


setup(
    name='savepagenow',
    version='1.0.0',
    description='A simple Python wrapper for archive.org\'s "Save Page Now" capturing service',
    long_description=read('README.rst'),
    author='Ben Welsh',
    author_email='ben.welsh@gmail.com',
    url='https://www.github.com/pastpages/savepagenow/',
    packages=('savepagenow',),
    include_package_data=True,
    license="MIT",
    install_requires=[
        'requests>=2.20.0',
        'click'
    ],
    entry_points='''
        [console_scripts]
        savepagenow=savepagenow.api:cli
    ''',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: MIT License'
    ],
    project_urls={
        'Project': 'http://www.pastpages.org/',
        'Source': 'https://github.com/pastpages/savepagenow',
        'Tracker': 'https://github.com/pastpages/savepagenow/issues'
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import click
import requests
from .exceptions import (
    CachedPage,
    WaybackRuntimeError,
    BlockedByRobots
)
from urllib.parse import urljoin
from requests.utils import parse_header_links


def capture(
    target_url,
    user_agent="savepagenow (https://github.com/pastpages/savepagenow)",
    accept_cache=False
):
    """
    Archives the provided URL using archive.org's Wayback Machine.

    Returns the archive.org URL where the capture is stored.

    Raises a CachedPage exception if archive.org declines to conduct a new
    capture and returns a previous snapshot instead.

    To silence that exception, pass into True to the ``accept_cache`` keyword
    argument.
    """
    # Put together the URL that will save our request
    domain = "https://web.archive.org"
    save_url = urljoin(domain, "/save/")
    request_url = save_url + target_url

    # Send the capture request to archive.org
    headers = {
        'User-Agent': user_agent,
    }
    response = requests.get(request_url, headers=headers)

    # If it has an error header, raise that.
    has_error_header = 'X-Archive-Wayback-Runtime-Error' in response.headers
    if has_error_header:
        error_header = response.headers['X-Archive-Wayback-Runtime-Error']
        if error_header == 'RobotAccessControlException: Blocked By Robots':
            raise BlockedByRobots("archive.org returned blocked by robots.txt error")
        else:
            raise WaybackRuntimeError(error_header)

    # If it has an error code, raise that
    if response.status_code in [403, 502, 520]:
        raise WaybackRuntimeError(response.headers)

    # Parse the Link tag in the header, which points to memento URLs in Wayback
    header_links = parse_header_links(response.headers['Link'])

    # The link object marked as the `memento` is the one we want to return
    try:
        archive_obj = [h for h in header_links if h['rel'] == 'memento'][0]
        archive_url = archive_obj['url']
    except (IndexError, KeyError):
        raise WaybackRuntimeError(dict(status_code=response.status_code, headers=response.headers))

    # Determine if the response was cached
    cached = 'X-Page-Cache' in response.headers and response.headers['X-Page-Cache'] == 'HIT'

    # If it was cached ...
    if cached:
        # .. and we're not allowing that
        if not accept_cache:
            # ... throw an error
            msg = "archive.org returned a cache of this page: {}".format(archive_url)
            raise CachedPage(msg)

    # Finally, return the archived URL
    return archive_url


def capture_or_cache(
    target_url,
    user_agent="savepagenow (https://github.com/pastpages/savepagenow)"
):
    """
    Archives the provided URL using archive.org's Wayback Machine, unless
    the page has been recently captured.

    Returns a tuple with the archive.org URL where the capture is stored,
    along with a boolean indicating if a new capture was conducted.

    If the boolean is True, archive.org conducted a new capture. If it is False,
    archive.org has returned a recently cached capture instead, likely taken
    in the previous minutes.
    """
    try:
        return capture(target_url, user_agent=user_agent, accept_cache=False), True
    except CachedPage:
        return capture(target_url, user_agent=user_agent, accept_cache=True), False


@click.command()
@click.argument("url")
@click.option("-ua", "--user-agent", help="User-Agent header for the web request")
@click.option("-c", "--accept-cache", help="Accept and return cached URL", is_flag=True)
def cli(url, user_agent, accept_cache):
    """
    Archives the provided URL using archive.org's Wayback Machine.

    Raises a CachedPage exception if archive.org declines to conduct a new
    capture and returns a previous snapshot instead.
    """
    kwargs = {}
    if user_agent:
        kwargs['user_agent'] = user_agent
    if accept_cache:
        kwargs['accept_cache'] = accept_cache
    archive_url = capture(url, **kwargs)
    click.echo(archive_url)


if __name__ == "__main__":
    cli()
